#!/usr/bin/env python
#
# grib_checker.py checks Checks whatever you want to check in your life
#
# AUTHOR: BSC-CNS I. CERNICHARO
#

import sys
import numpy as np
from eccodes import GribFile

import data_checks
import metadata_checks

class grib_content:
    """grib_content object class set a list of checks and properties where
    the content of a grib file will be evaluated. All it needs to initiate
    is the file path of the GRIB file we will evaluate
    """
    def __init__(self,filename):
        self.name = filename
        self.grib_content = GribFile(self.name)
        self.pname = filename.split("/")[-1]

    def print_num_keys(self):
        # It returns the file name without the path
        return self.pname+" has "+str(len(self.grib_content))+" messages"

    def check_data_coords(self):
        # The following lists will be filled with the log report for each
        # message. Coordinates and data values will be tested
        coords_log = list()
        data_log = list()
        index = list()

        # Checking all messages. Here we call methods that are imported from
        # data_checks submodule
        for i,msg in enumerate(self.grib_content):
            coords_log.append(data_checks.check_coords(msg))
            data_log.append(data_checks.check_data_values(msg))
            index.append(i)
        
        err_coords = coords_log.count("ERROR")
        err_data = data_log.count("ERROR")

        if err_coords != 0:
            msg_coord = chr(27)+"[0;31m"+"\nOne or more messages have incorrect data or missing keys"+chr(27)+"[0m"
        else:
            msg_coord = chr(27)+"[0;32m"+"\nEverything OK with coordinates"+chr(27)+"[0m"

        if err_data != 0:
            msg_data = chr(27)+"[0;31m"+"\nOne or more messages have incorrect data or missing keys"+chr(27)+"[0m"
        else:
            msg_data = chr(27)+"[0;32m"+"\nEverything OK with data values"+chr(27)+"[0m"

        return msg_data,msg_coord

    def check_meta_grid(self):
        # The following lists will be filled with the log report for each
        # message. Metadata of the header and of the grid description section
        # will be tested.
        meta_log = list()
        grid_log = list()
        index = list()
        # These lists, g_ed and grid_t will check the homogeneity of the 
        # messages for the GRIBEditionNumber and gridType keys.
        g_ed = list()
        grid_t = list()
        missing_meta = list()
        missing_grid = list()

        # Checking all messages. Here we call methods that are imported from
        # metadata_checks submodule
        for i,msg in enumerate(self.grib_content):
            index.append(i)

            g_ed.append(msg["GRIBEditionNumber"])
            grid_t.append(msg["gridType"])

            meta_cck = metadata_checks.check_metada_keys(msg)
            grid_cck = metadata_checks.check_grid_keys(msg)

            if len(meta_cck) > 1:
                for item in meta_cck[1]:
                    if item not in missing_meta:
                        missing_meta.append(item)

            if len(grid_cck) > 1:
                for item in grid_cck[1]:
                    if item not in missing_grid:
                        missing_grid.append(item)

            meta_log.append(meta_cck[0])
            grid_log.append(grid_cck[0])

        grib_editions = list(set(g_ed))
        grid_types = list(set(grid_t))
        
        err_meta = meta_log.count("ERROR")
        err_grid = grid_log.count("ERROR")

        if len(grib_editions) > 1:
            msg_editions = chr(27)+"[0;31m"+"\nGrib Edition Check:\n\tFound different Grib editions in the messages\n"+chr(27)+"[0m"
            for item in grib_editions:
                msg_editions = msg_editions + "\t\tEdition: " + str(item) + "\n"
        else:
            msg_editions = chr(27)+"[0;32m"+"\nGrib Edition Check:\n\tEverything OK, one unique edition found"+chr(27)+"[0m"
            msg_editions += "\n\t\tEdition: " + str(grib_editions[0])

        if len(grid_types) > 1:
            msg_grid_type = chr(27)+"[0;31m"+"\nGrid Type Check:\n\tFound different Grib editions in the messages\n"+chr(27)+"[0m"
            for item in grid_types:
                msg_grid_type = msg_grid_type + "\t\tGrid type: " + str(item) + "\n"
        else:
            msg_grid_type = chr(27)+"[0;32m"+"\nGrid Type Check:\n\tEverything OK, one unique edition found"+chr(27)+"[0m"
            msg_grid_type += "\n\t\tGrid type: " + str(grid_types[0])

        if err_meta != 0:
            msg_meta = chr(27)+"[0;31m"+"\nMetadata Check:\n\tOne or more messages have incorrect or missing KEYS\n"+chr(27)+"[0m"
            for item in missing_meta:
                msg_meta += "\t\tKey: " + str(item) + "\n"
        else:
            msg_meta = chr(27)+"[0;32m"+"\nMetadata Check:\n\tEverything OK"+chr(27)+"[0m"

        if err_grid != 0:
            msg_grid = chr(27)+"[0;31m"+"\nGrid Metada Check:\n\tOne or more messages have incorrect or missing Keys\n"+chr(27)+"[0m"
            for item in missing_grid:
                msg_grid += "\t\tKey: " + str(item) + "\n"
        else:
            msg_grid = chr(27)+"[0;32m"+"\nGrid Metada Check:\n\tEverything OK with data values"+chr(27)+"[0m"

        return msg_meta,msg_grid,msg_grid_type,msg_editions

        # STILL WORKING ON THIS, and YES, I AM YELLING
